@@ -31,11 +31,11 @@ class Plugin(object):
 
 		self._instances = collections.OrderedDict()
 		self._init_commands()
-		self._init_devices()
 
 		self._global_cfg = global_cfg
 		self._variables = variables
 		self._has_dynamic_options = False
+		self._devices_inited = False
 
 		self._options_used_by_dynamic = self._get_config_options_used_by_dynamic()
 
@@ -44,6 +44,11 @@ class Plugin(object):
 	def cleanup(self):
 		self.destroy_instances()
 
+	def init_devices(self):
+		if not self._devices_inited:
+			self._init_devices()
+			self._devices_inited = True
+
 	@property
 	def name(self):
 		return self.__class__.__module__.split(".")[-1].split("_", 1)[1]
@@ -57,6 +62,11 @@ class Plugin(object):
 		"""Default configuration options for the plugin."""
 		return {}
 
+	@classmethod
+	def get_config_options_hints(cls):
+		"""Explanation of each config option function"""
+		return {}
+
 	@classmethod
 	def _get_config_options_used_by_dynamic(self):
 		"""List of config options used by dynamic tuning. Their previous values will be automatically saved and restored."""
@@ -113,7 +123,7 @@ class Plugin(object):
 
 	def destroy_instances(self):
 		"""Destroy all instances."""
-		for instance in self._instances.values():
+		for instance in list(self._instances.values()):
 			log.debug("destroying instance %s (%s)" % (instance.name, self.name))
 			self._destroy_instance(instance)
 		self._instances.clear()
@@ -151,7 +161,7 @@ class Plugin(object):
 				log.error("Plugin '%s' does not support the 'devices_udev_regex' option", self.name)
 				return set()
 			udev_devices = self._device_matcher_udev.match_list(instance.devices_udev_regex, udev_devices)
-			return set(map(lambda x: x.sys_name, udev_devices))
+			return set([x.sys_name for x in udev_devices])
 
 	def assign_free_devices(self, instance):
 		if not self._devices_supported:
@@ -167,7 +177,7 @@ class Plugin(object):
 			if instance.name != self.name:
 				name += " (%s)" % self.name
 			log.info("instance %s: assigning devices %s" % (name, ", ".join(to_assign)))
-			instance.devices.update(to_assign) # cannot use |=
+			instance.assigned_devices.update(to_assign) # cannot use |=
 			self._assigned_devices |= to_assign
 			self._free_devices -= to_assign
 
@@ -175,10 +185,13 @@ class Plugin(object):
 		if not self._devices_supported:
 			return
 
-		to_release = instance.devices & self._assigned_devices
+		to_release = (instance.processed_devices \
+				| instance.assigned_devices) \
+				& self._assigned_devices
 
 		instance.active = False
-		instance.devices.clear()
+		instance.processed_devices.clear()
+		instance.assigned_devices.clear()
 		self._assigned_devices -= to_release
 		self._free_devices |= to_release
 
@@ -186,10 +199,8 @@ class Plugin(object):
 	# Tuning activation and deactivation.
 	#
 
-	def _run_for_each_device(self, instance, callback):
-		if self._devices_supported:
-			devices = instance.devices
-		else:
+	def _run_for_each_device(self, instance, callback, devices):
+		if not self._devices_supported:
 			devices = [None, ]
 
 		for device in devices:
@@ -201,7 +212,7 @@ class Plugin(object):
 	def _instance_post_static(self, instance, enabling):
 		pass
 
-	def _call_device_script(self, instance, script, op, devices, full_rollback = False):
+	def _call_device_script(self, instance, script, op, devices, rollback = consts.ROLLBACK_SOFT):
 		if script is None:
 			return None
 		if len(devices) == 0:
@@ -217,14 +228,16 @@ class Plugin(object):
 			environ = os.environ
 			environ.update(self._variables.get_env())
 			arguments = [op]
-			if full_rollback:
+			if rollback == consts.ROLLBACK_FULL:
 				arguments.append("full_rollback")
 			arguments.append(dev)
 			log.info("calling script '%s' with arguments '%s'" % (script, str(arguments)))
-			log.debug("using environment '%s'" % str(environ.items()))
+			log.debug("using environment '%s'" % str(list(environ.items())))
 			try:
-				proc = Popen([script] +  arguments, stdout=PIPE, stderr=PIPE, close_fds=True, env=environ, \
-					cwd = dir_name)
+				proc = Popen([script] +  arguments, \
+						stdout=PIPE, stderr=PIPE, \
+						close_fds=True, env=environ, \
+						cwd = dir_name, universal_newlines = True)
 				out, err = proc.communicate()
 				if proc.returncode:
 					log.error("script '%s' error: %d, '%s'" % (script, proc.returncode, err[:-1]))
@@ -242,13 +255,17 @@ class Plugin(object):
 			return
 
 		if instance.has_static_tuning:
-			self._call_device_script(instance, instance.script_pre, "apply", instance.devices)
+			self._call_device_script(instance, instance.script_pre,
+					"apply", instance.assigned_devices)
 			self._instance_pre_static(instance, True)
 			self._instance_apply_static(instance)
 			self._instance_post_static(instance, True)
-			self._call_device_script(instance, instance.script_post, "apply", instance.devices)
+			self._call_device_script(instance, instance.script_post,
+					"apply", instance.assigned_devices)
 		if instance.has_dynamic_tuning and self._global_cfg.get(consts.CFG_DYNAMIC_TUNING, consts.CFG_DEF_DYNAMIC_TUNING):
-			self._run_for_each_device(instance, self._instance_apply_dynamic)
+			self._run_for_each_device(instance, self._instance_apply_dynamic, instance.assigned_devices)
+		instance.processed_devices.update(instance.assigned_devices)
+		instance.assigned_devices.clear()
 
 	def instance_verify_tuning(self, instance, ignore_missing):
 		"""
@@ -257,12 +274,16 @@ class Plugin(object):
 		if not instance.active:
 			return None
 
+		if len(instance.assigned_devices) != 0:
+			log.error("BUG: Some devices have not been tuned: %s"
+					% ", ".join(instance.assigned_devices))
+		devices = instance.processed_devices.copy()
 		if instance.has_static_tuning:
-			if self._call_device_script(instance, instance.script_pre, "verify", instance.devices) == False:
+			if self._call_device_script(instance, instance.script_pre, "verify", devices) == False:
 				return False
-			if self._instance_verify_static(instance, ignore_missing) == False:
+			if self._instance_verify_static(instance, ignore_missing, devices) == False:
 				return False
-			if self._call_device_script(instance, instance.script_post, "verify", instance.devices) == False:
+			if self._call_device_script(instance, instance.script_post, "verify", devices) == False:
 				return False
 			return True
 		else:
@@ -275,39 +296,45 @@ class Plugin(object):
 		if not instance.active:
 			return
 		if instance.has_dynamic_tuning and self._global_cfg.get(consts.CFG_DYNAMIC_TUNING, consts.CFG_DEF_DYNAMIC_TUNING):
-			self._run_for_each_device(instance, self._instance_update_dynamic)
+			self._run_for_each_device(instance, self._instance_update_dynamic, instance.processed_devices.copy())
 
-	def instance_unapply_tuning(self, instance, full_rollback = False):
+	def instance_unapply_tuning(self, instance, rollback = consts.ROLLBACK_SOFT):
 		"""
 		Remove all tunings applied by the plugin instance.
 		"""
+		if rollback == consts.ROLLBACK_NONE:
+			return
+
 		if instance.has_dynamic_tuning and self._global_cfg.get(consts.CFG_DYNAMIC_TUNING, consts.CFG_DEF_DYNAMIC_TUNING):
-			self._run_for_each_device(instance, self._instance_unapply_dynamic)
+			self._run_for_each_device(instance, self._instance_unapply_dynamic, instance.processed_devices)
 		if instance.has_static_tuning:
-			self._call_device_script(instance, instance.script_post, "unapply", instance.devices, full_rollback = full_rollback)
+			self._call_device_script(instance, instance.script_post,
+					"unapply", instance.processed_devices,
+					rollback = rollback)
 			self._instance_pre_static(instance, False)
-			self._instance_unapply_static(instance, full_rollback)
+			self._instance_unapply_static(instance, rollback)
 			self._instance_post_static(instance, False)
-			self._call_device_script(instance, instance.script_pre, "unapply", instance.devices, full_rollback = full_rollback)
+			self._call_device_script(instance, instance.script_pre, "unapply", instance.processed_devices, rollback = rollback)
 
 	def _instance_apply_static(self, instance):
 		self._execute_all_non_device_commands(instance)
-		self._execute_all_device_commands(instance, instance.devices)
+		self._execute_all_device_commands(instance, instance.assigned_devices)
 
-	def _instance_verify_static(self, instance, ignore_missing):
+	def _instance_verify_static(self, instance, ignore_missing, devices):
 		ret = True
 		if self._verify_all_non_device_commands(instance, ignore_missing) == False:
 			ret = False
-		if self._verify_all_device_commands(instance, instance.devices, ignore_missing) == False:
+		if self._verify_all_device_commands(instance, devices, ignore_missing) == False:
 			ret = False
 		return ret
 
-	def _instance_unapply_static(self, instance, full_rollback = False):
-		self._cleanup_all_device_commands(instance, instance.devices)
+	def _instance_unapply_static(self, instance, rollback = consts.ROLLBACK_SOFT):
+		self._cleanup_all_device_commands(instance,
+				instance.processed_devices)
 		self._cleanup_all_non_device_commands(instance)
 
 	def _instance_apply_dynamic(self, instance, device):
-		for option in filter(lambda opt: self._storage_get(instance, self._commands[opt], device) is None, self._options_used_by_dynamic):
+		for option in [opt for opt in self._options_used_by_dynamic if self._storage_get(instance, self._commands[opt], device) is None]:
 			self._check_and_save_value(instance, self._commands[option], device)
 
 		self._instance_update_dynamic(instance, device)
@@ -359,13 +386,13 @@ class Plugin(object):
 			self._commands[command_name] = info
 
 		# sort commands by priority
-		self._commands = collections.OrderedDict(sorted(self._commands.iteritems(), key=lambda (name, info): info["priority"]))
+		self._commands = collections.OrderedDict(sorted(iter(self._commands.items()), key=lambda name_info: name_info[1]["priority"]))
 
 	def _check_commands(self):
 		"""
 		Check if all commands are defined correctly.
 		"""
-		for command_name, command in self._commands.items():
+		for command_name, command in list(self._commands.items()):
 			# do not check custom commands
 			if command.get("custom", False):
 				continue
@@ -377,11 +404,14 @@ class Plugin(object):
 	# Operations with persistent storage for status data.
 	#
 
-	def _storage_key(self, instance_name, command_name, device_name=None):
-		if device_name is not None:
-			return "%s/%s/%s" % (command_name, instance_name, device_name)
-		else:
-			return "%s/%s" % (command_name, instance_name)
+	def _storage_key(self, instance_name = None, command_name = None,
+			device_name = None):
+		class_name = type(self).__name__
+		instance_name = "" if instance_name is None else instance_name
+		command_name = "" if command_name is None else command_name
+		device_name = "" if device_name is None else device_name
+		return "%s/%s/%s/%s" % (class_name, instance_name,
+				command_name, device_name)
 
 	def _storage_set(self, instance, command, value, device_name=None):
 		key = self._storage_key(instance.name, command["name"], device_name)
@@ -400,13 +430,13 @@ class Plugin(object):
 	#
 
 	def _execute_all_non_device_commands(self, instance):
-		for command in filter(lambda command: not command["per_device"], self._commands.values()):
+		for command in [command for command in list(self._commands.values()) if not command["per_device"]]:
 			new_value = self._variables.expand(instance.options.get(command["name"], None))
 			if new_value is not None:
 				self._execute_non_device_command(instance, command, new_value)
 
 	def _execute_all_device_commands(self, instance, devices):
-		for command in filter(lambda command: command["per_device"], self._commands.values()):
+		for command in [command for command in list(self._commands.values()) if command["per_device"]]:
 			new_value = self._variables.expand(instance.options.get(command["name"], None))
 			if new_value is None:
 				continue
@@ -415,7 +445,7 @@ class Plugin(object):
 
 	def _verify_all_non_device_commands(self, instance, ignore_missing):
 		ret = True
-		for command in filter(lambda command: not command["per_device"], self._commands.values()):
+		for command in [command for command in list(self._commands.values()) if not command["per_device"]]:
 			new_value = self._variables.expand(instance.options.get(command["name"], None))
 			if new_value is not None:
 				if self._verify_non_device_command(instance, command, new_value, ignore_missing) == False:
@@ -424,7 +454,7 @@ class Plugin(object):
 
 	def _verify_all_device_commands(self, instance, devices, ignore_missing):
 		ret = True
-		for command in filter(lambda command: command["per_device"], self._commands.values()):
+		for command in [command for command in list(self._commands.values()) if command["per_device"]]:
 			new_value = instance.options.get(command["name"], None)
 			if new_value is None:
 				continue
@@ -488,8 +518,8 @@ class Plugin(object):
 
 	def _norm_value(self, value):
 		v = self._cmd.unquote(str(value))
-		if re.match(r'\s*(0+,)+[\da-fA-F]*\s*$', v):
-			return re.sub(r'^\s*(0+,)+', "", v)
+		if re.match(r'\s*(0+,?)+([\da-fA-F]*,?)*\s*$', v):
+			return re.sub(r'^\s*(0+,?)+', "", v)
 		return v
 
 	def _verify_value(self, name, new_value, current_value, ignore_missing, device = None):
@@ -513,7 +543,20 @@ class Plugin(object):
 					ret = int(new_value, 16) == int(current_value, 16)
 				except ValueError:
 					ret = str(new_value) == str(current_value)
-		if ret:
+					if not ret:
+						vals = str(new_value).split('|')
+						for val in vals:
+							val = val.strip()
+							ret = val == current_value
+							if ret:
+								break
+		self._log_verification_result(name, ret, new_value,
+				current_value, device = device)
+		return ret
+
+	def _log_verification_result(self, name, success, new_value,
+			current_value, device = None):
+		if success:
 			if device is None:
 				log.info(consts.STR_VERIFY_PROFILE_VALUE_OK % (name, str(current_value).strip()))
 			else:
@@ -547,12 +590,12 @@ class Plugin(object):
 		return self._verify_value(command["name"], new_value, current_value, ignore_missing)
 
 	def _cleanup_all_non_device_commands(self, instance):
-		for command in reversed(filter(lambda command: not command["per_device"], self._commands.values())):
+		for command in reversed([command for command in list(self._commands.values()) if not command["per_device"]]):
 			if (instance.options.get(command["name"], None) is not None) or (command["name"] in self._options_used_by_dynamic):
 				self._cleanup_non_device_command(instance, command)
 
 	def _cleanup_all_device_commands(self, instance, devices):
-		for command in reversed(filter(lambda command: command["per_device"], self._commands.values())):
+		for command in reversed([command for command in list(self._commands.values()) if command["per_device"]]):
 			if (instance.options.get(command["name"], None) is not None) or (command["name"] in self._options_used_by_dynamic):
 				for device in devices:
 					self._cleanup_device_command(instance, command, device)
