@@ -1,7 +1,7 @@
 import re
 import os.path
-import base
-from decorators import *
+from . import base
+from .decorators import *
 import tuned.logs
 from subprocess import *
 from tuned.utils.commands import commands
@@ -11,11 +11,45 @@ log = tuned.logs.get()
 
 class ModulesPlugin(base.Plugin):
 	"""
-	Plugin for applying custom kernel modules options.
+	`modules`::
+	
+	Plug-in for applying custom kernel modules options.
+	+
+	This plug-in can set parameters to kernel modules. It creates
+	`/etc/modprobe.d/tuned.conf` file. The syntax is
+	`_module_=_option1=value1 option2=value2..._` where `_module_` is
+	the module name and `_optionx=valuex_` are module options which may
+	or may not be present.
+	+
+	.Load module `netrom` with module parameter `nr_ndevs=2`
+	====
+	----
+	[modules]
+	netrom=nr_ndevs=2
+	----
+	====
+	Modules can also be forced to load/reload by using an additional
+	`+r` option prefix.
+	+
+	.(Re)load module `netrom` with module parameter `nr_ndevs=2`
+	====
+	----
+	[modules]
+	netrom=+r nr_ndevs=2
+	----
+	====
+	The `+r` switch will also cause *TuneD* to try and remove `netrom`
+	module (if loaded) and try and (re)insert it with the specified
+	parameters. The `+r` can be followed by an optional comma (`+r,`)
+	for better readability.
+	+
+	When using `+r` the module will be loaded immediately by the *TuneD*
+	daemon itself rather than waiting for the OS to load it with the
+	specified parameters.
 	"""
 
 	def __init__(self, *args, **kwargs):
-		super(self.__class__, self).__init__(*args, **kwargs)
+		super(ModulesPlugin, self).__init__(*args, **kwargs)
 		self._has_dynamic_options = True
 		self._cmd = commands()
 
@@ -45,7 +79,7 @@ class ModulesPlugin(base.Plugin):
 		retcode = 0
 		skip_check = False
 		reload_list = []
-		for option, value in instance._modules.items():
+		for option, value in list(instance._modules.items()):
 			module = self._variables.expand(option)
 			v = self._variables.expand(value)
 			if not skip_check:
@@ -73,12 +107,12 @@ class ModulesPlugin(base.Plugin):
 	def _unquote_path(self, path):
 		return str(path).replace("/", "")
 
-	def _instance_verify_static(self, instance, ignore_missing):
+	def _instance_verify_static(self, instance, ignore_missing, devices):
 		ret = True
 		# not all modules exports all their parameteters through sysfs, so hardcode check with ignore_missing
 		ignore_missing = True
 		r = re.compile(r"\s+")
-		for option, value in instance._modules.items():
+		for option, value in list(instance._modules.items()):
 			module = self._variables.expand(option)
 			v = self._variables.expand(value)
 			v = re.sub(r"^\s*\+r\s*,?\s*", "", v)
@@ -100,8 +134,8 @@ class ModulesPlugin(base.Plugin):
 								ret = False
 		return ret
 
-	def _instance_unapply_static(self, instance, full_rollback = False):
-		if full_rollback:
+	def _instance_unapply_static(self, instance, rollback = consts.ROLLBACK_SOFT):
+		if rollback == consts.ROLLBACK_FULL:
 			self._clear_modprobe_file()
 
 	def _clear_modprobe_file(self):
